------------------------------------------------------------------------
--! @file GlobalModule.vhd
--! @brief <b>Global Clock and reset Generator </b><p>
--!   Generates the internal clock and reset signals
--!   Note: oDsSampleClk20M0, oDsSampleClk10M0 and oDsSampleClk0M0 have to be phase synchroneous
--!         whith the rising edge at the same time. Required for DeltaSigmaClk vs. SampleClk at inputLatch
--! </p>
--! <p>
--! <b>Source:</b><p>
------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.fpga_base_modules_pack.all;

entity GlobalModule is
    generic(
        gCmpPwmMax          : integer   := 64000;
        gCmpPwmWidth        : integer   := 16
    );
    port(
        iExtRstRXN          : in  std_logic;                    --! Reset
        iExtClk             : in  std_logic;                    --! External Clock
        oSysRst             : out std_logic;                    --! System Reset
        oFastClk            : out std_logic;                    --! Fast Clock 100 MHz
        oSysClk             : out std_logic;                    --! System Clock 50 MHz
        oCmpPwmCnt          : out std_logic_vector(gCmpPwmWidth -1 downto 0); --! Reference PWM
        oDsSampleClk25M0    : out std_logic;                    --! Internal DeltaSigma Clock 25 MHz for sampling the Delta Sigma Data Stream
        oDsSampleClk20M0    : out std_logic;                    --! Internal DeltaSigma Clock 20 MHz for sampling the Delta Sigma Data Stream
        oDsSampleClk10M0    : out std_logic;                    --! Internal DeltaSigma Clock 10 MHz for sampling the Delta Sigma Data Stream
        oDsSampleClk5M0     : out std_logic;                    --! Internal DeltaSigma Clock 5 MHz for sampling the Delta Sigma Data Stream
        oDsClk25M0Oddr      : out std_logic_vector(1 downto 0); --! external Delta Sigma Clock Oddr 25 MHz. Connect directly to ODDR Register
        oDsClk20M0Oddr      : out std_logic_vector(1 downto 0); --! external Delta Sigma Clock Oddr 20 MHz. Connect directly to ODDR Register
        oDsClk10M0Oddr      : out std_logic_vector(1 downto 0); --! external Delta Sigma Clock Oddr 10 MHz. Connect directly to ODDR Register
        oDsClk5M0Oddr       : out std_logic_vector(1 downto 0)  --! external Delta Sigma Clock Oddr 5 MHz. Connect directly to ODDR Register
    );
end GlobalModule;

architecture RTL of GlobalModule is

    signal sPllReset100     : std_logic;
    signal sPllLock100      : std_logic;
    signal sSysClk          : std_logic;
    signal sFastClk         : std_logic;
    signal sSysRst          : std_logic;
    signal sSysRstSyn       : std_logic_vector(2 downto 0);

    signal sCmpPwmCnt       : unsigned(gCmpPwmWidth -1 downto 0);

    signal sClk25M0Cnt      : unsigned(1 downto 0);
    signal sClk25M0Sreg     : std_logic_vector(3 downto 0);
    signal sClkCnt          : unsigned(4 downto 0);

begin



    -- Generate 25 MHz DeltaSigma Clock in logic
    plDS25M0ClkGen : process(sSysRst, sFastClk)
    begin
        if sSysRst = '1' then
            sClk25M0Cnt  <= "11";   --start with "11" to synchronize oDsSampleClk25M0 and the ODDR Output
            sClk25M0Sreg <= "1100";
        elsif rising_edge(sFastClk) then
            sClk25M0Cnt  <= sClk25M0Cnt + 1;
            sClk25M0Sreg <= sClk25M0Sreg(0) & sClk25M0Sreg(3 downto 1);
        end if;
    end process;
    oDsClk25M0Oddr   <= sClk25M0Sreg(1 downto 0);    
    oDsSampleClk25M0 <= sClk25M0Cnt(1);



    -- Clock Counter used for SampleClk and OddrClk with frequencies of 20M, 10M and 5M
    plCntGen : process(sSysRst, sFastClk)
    begin
        if sSysRst = '1' then
            sClkCnt         <= "00000";
        elsif rising_edge(sFastClk) then
            sClkCnt <= sClkCnt +1;
            if sClkCnt = 19 then
                sClkCnt <= "00000";
            end if;
        end if;
    end process;


    -- Generate internal DeltaSigma SampleClock in logic
    plDSSampleClkGen : process(sSysRst, sFastClk)
    begin
        if sSysRst = '1' then
            oDsSampleClk20M0    <= '0';
            oDsSampleClk10M0    <= '0';
            oDsSampleClk5M0     <= '0';
        elsif rising_edge(sFastClk) then
            case sClkCnt is
                when "00000" => oDsSampleClk20M0 <= '1';    -- 0
                                oDsSampleClk10M0 <= '1';
                                oDsSampleClk5M0  <= '1';
                when "00010" => oDsSampleClk20M0 <= '0';    -- 2
                when "00101" => oDsSampleClk20M0 <= '1';    -- 5
                                oDsSampleClk10M0 <= '0';
                when "00111" => oDsSampleClk20M0 <= '0';    -- 7
                when "01010" => oDsSampleClk20M0 <= '1';    -- 10
                                oDsSampleClk10M0 <= '1';
                                oDsSampleClk5M0  <= '0';
                when "01100" => oDsSampleClk20M0 <= '0';    -- 12
                when "01111" => oDsSampleClk20M0 <= '1';    -- 15
                                oDsSampleClk10M0 <= '0';
                when "10001" => oDsSampleClk20M0 <= '0';    -- 17
                when others =>  NULL;
            end case;
        end if;
    end process;

    -- Generate external DeltaSigma ODDR Clk
    plDSOddrClkGen : process(sSysRst, sFastClk)
    begin
        if sSysRst = '1' then
            oDsClk20M0Oddr <= "00";
            oDsClk10M0Oddr <= "00";
            oDsClk5M0Oddr  <= "00";
        elsif rising_edge(sFastClk) then
            case sClkCnt is
                when "00000" => oDsClk20M0Oddr <= "11"; -- 0
                                oDsClk10M0Oddr <= "11";
                                oDsClk5M0Oddr  <= "11";
                when "00010" => oDsClk20M0Oddr <= "00"; -- 2
                when "00100" => oDsClk20M0Oddr <= "10"; -- 4
                                oDsClk10M0Oddr <= "01";
                when "00101" => oDsClk20M0Oddr <= "11"; -- 5
                                oDsClk10M0Oddr <= "00";
                when "00111" => oDsClk20M0Oddr <= "00"; -- 7
                when "01001" => oDsClk20M0Oddr <= "10"; -- 9
                                oDsClk10M0Oddr <= "10";
                                oDsClk5M0Oddr  <= "01";
                when "01010" => oDsClk20M0Oddr <= "11"; -- 10
                                oDsClk10M0Oddr <= "11";
                                oDsClk5M0Oddr  <= "00";
                when "01100" => oDsClk20M0Oddr <= "00"; -- 12
                when "01110" => oDsClk20M0Oddr <= "10"; -- 14
                                oDsClk10M0Oddr <= "01";
                when "01111" => oDsClk20M0Oddr <= "11"; -- 15
                                oDsClk10M0Oddr <= "00";
                when "10001" => oDsClk20M0Oddr <= "00"; -- 17
                when "10011" => oDsClk20M0Oddr <= "10"; -- 19
                                oDsClk10M0Oddr <= "10";
                                oDsClk5M0Oddr  <= "10";
                when others =>  NULL;
            end case;
        end if;
    end process;



    -- System clocks (100.0 MHz; 50.0 MHz) + Delta Sigma Clock 25.0 MHz
    sPllReset100 <= not iExtRstRXN; -- use external Reset to reset the PLL100M0
    ilPll100M0 : PLL100M0
    port map (
        CLK     => iExtClk,         -- external clock "ExtClk"
        RESET   => sPllReset100,    -- External Reset
        CLKOP   => sSysClk,         -- normal internal system clock (= 50.0 MHz)
        CLKOS   => sFastClk,        -- fast internal system clock (= 100.0 MHz)
        LOCK    => sPllLock100      -- PLL Lock Signal, used for PLL20M0 reset
    );


    -- Use PLL Lock10 for reset generation to make sure that both cascaded PLLs are locked
    plResetGeneration : process (sSysClk, sPllLock100)
    begin
        if sPllLock100 = '0' then
            sSysRstSyn  <= (others => '1');
            sSysRst     <= '1';
        elsif rising_edge(sSysClk) then
            sSysRstSyn  <= sSysRstSyn(sSysRstSyn'high-1 downto 0) & not sPllLock100;
            sSysRst     <= sSysRstSyn(sSysRstSyn'high);
        end if;
    end process;


    plReferencePwmCnt : process(sSysRst, sSysClk)
    begin
        if sSysRst = '1' then
            sCmpPwmCnt  <= (others => '0');
        elsif rising_edge(sSysClk) then
            if sCmpPwmCnt = gCmpPwmMax then
                sCmpPwmCnt <= (others => '0');
            else
                sCmpPwmCnt <= sCmpPwmCnt + 1;
            end if;
        end if;
    end process;

    oCmpPwmCnt  <= std_logic_vector(sCmpPwmCnt);
    
    -- Output Assignment
    oSysRst     <= sSysRst;
    oSysClk     <= sSysClk;
    oFastClk    <= sFastClk;

end RTL;