------------------------------------------------------------------------
--! @file top.vhd
--! @brief <b>Top level for Demonstration of ODDR Simulation Error</b><p>
------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.fpga_base_modules_pack.all;

entity Top is
    port (
        iExtRstN        : in std_logic;
        iExtClk         : in std_logic
  );
end Top;

architecture Behavior of Top is
-- ***********************************************
--  Debug only signals
-- ***********************************************

-- ***********************************************

    signal sReset                   : std_logic;        -- Global Reset Signal
    -- clock signals
    signal sSysClk                  : std_logic;        -- PLL Generated System clock (50.0 Mhz) for internal use
    signal sFastClk                 : std_logic;        -- Fast PLL Generated System clock (100.0 Mhz) for internal use
    
    signal sReferenceClk_25         : std_logic;        -- Fast Delta Sigma Clock (25.0 Mhz) for input sampling
    signal sReferenceClk_20         : std_logic;        -- Fast Delta Sigma Clock (20.0 Mhz) for input sampling
    signal sReferenceClk_10         : std_logic;        -- Slow Delta Sigma Clock (10.0 Mhz) for input sampling (reduced datarate)
    signal sReferenceClk_5          : std_logic;        -- Slow Delta Sigma Clock (5.0 Mhz) for input sampling(reduced datarate)

    signal sDirectOddrInputBits_25           : std_logic_vector(1 downto 0);     -- Fast Delta Sigma Clock (25.0 Mhz) for external Delta-Sigma-ADC, Connect to ODDR FlipFlop
    signal sDirectOddrInputBits_20           : std_logic_vector(1 downto 0);     -- Fast Delta Sigma Clock (20.0 Mhz) for external Delta-Sigma-ADC, Connect to ODDR FlipFlop
    signal sDirectOddrInputBits_10           : std_logic_vector(1 downto 0);     -- Slow Delta Sigma Clock (10.0 Mhz) for external Delta-Sigma-ADC, Connect to ODDR FlipFlop
    signal sDirectOddrInputBits_5            : std_logic_vector(1 downto 0);     -- Slow Delta Sigma Clock (5.0 Mhz) for external Delta-Sigma-ADC, Connect to ODDR FlipFlop

    signal sAbstractedOddrInputBits_25     : std_logic_vector(1 downto 0);                         --! external Delta Sigma Clock, to be connected to ODDR
    signal sAbstractedOddrInputBits_20     : std_logic_vector(1 downto 0);                         -- external Delta Sigma Clock, to be connected to ODDR
    signal sAbstractedOddrInputBits_10     : std_logic_vector(1 downto 0);                         -- external Delta Sigma Clock, to be connected to ODDR
    signal sAbstractedOddrInputBits_5      : std_logic_vector(1 downto 0);                         -- external Delta Sigma Clock, to be connected to ODDR
    
    
    signal sOddrOutDirect_25        : std_logic;
    signal sOddrOutAbstract_25      : std_logic;
    
    signal sOddrOutDirect_20        : std_logic;
    signal sOddrOutAbstract_20      : std_logic;

    signal sOddrOutDirect_10        : std_logic;
    signal sOddrOutAbstract_10      : std_logic;

    signal sOddrOutDirect_5        : std_logic;
    signal sOddrOutAbstract_5      : std_logic;
    
-- **************************************************

begin



    -- component assignments
    ilGlobalModule : GlobalModule
    generic map(
        gCmpPwmMax      => 64000,
        gCmpPwmWidth    => 16
    )
    port map(
        iExtRstRXN      => iExtRstN,
        iExtClk         => iExtClk,
        oSysRst         => sReset,
        oFastClk        => sFastClk,
        oSysClk         => sSysClk,
        oCmpPwmCnt      => open,
        oDsSampleClk25M0=> sReferenceClk_25,
        oDsSampleClk20M0=> sReferenceClk_20,
        oDsSampleClk10M0=> sReferenceClk_10,
        oDsSampleClk5M0 => sReferenceClk_5,
        oDsClk25M0Oddr  => sDirectOddrInputBits_25,
        oDsClk20M0Oddr  => sDirectOddrInputBits_20,
        oDsClk10M0Oddr  => sDirectOddrInputBits_10,
        oDsClk5M0Oddr   => sDirectOddrInputBits_5
    );





    sAbstractedOddrInputBits_25    <= sDirectOddrInputBits_25;
    sAbstractedOddrInputBits_20    <= sDirectOddrInputBits_20;
    sAbstractedOddrInputBits_10    <= sDirectOddrInputBits_10;
    sAbstractedOddrInputBits_5     <= sDirectOddrInputBits_5;
    

    


   ilOddr_25_direct_input: ODDR1
    port map (
        Clk     => sFastClk,
        Data    => sDirectOddrInputBits_25,
        Q(0)    => sOddrOutDirect_25
    );
    
    ilOddr_25_abstracted_input: ODDR1
    port map (
        Clk     => sFastClk,
        Data    => sAbstractedOddrInputBits_25,
        Q(0)    => sOddrOutAbstract_25
    );
    
    ilOddr_20_direct_input: ODDR1
    port map (
        Clk     => sFastClk,
        Data    => sDirectOddrInputBits_20,
        Q(0)    => sOddrOutDirect_20
    );
    
    ilOddr_20_abstracted_input: ODDR1
    port map (
        Clk     => sFastClk,
        Data    => sAbstractedOddrInputBits_20,
        Q(0)    => sOddrOutAbstract_20
    );
    
    ilOddr_10_direct_input: ODDR1
    port map (
        Clk     => sFastClk,
        Data    => sDirectOddrInputBits_10,
        Q(0)    => sOddrOutDirect_10
    );
    
    ilOddr_10_abstracted_input: ODDR1
    port map (
        Clk     => sFastClk,
        Data    => sAbstractedOddrInputBits_10,
        Q(0)    => sOddrOutAbstract_10
    );
    
    ilOddr_5_direct_input: ODDR1
    port map (
        Clk     => sFastClk,
        Data    => sDirectOddrInputBits_5,
        Q(0)    => sOddrOutDirect_5
    );
    
    ilOddr_5_abstracted_input: ODDR1
    port map (
        Clk     => sFastClk,
        Data    => sAbstractedOddrInputBits_5,
        Q(0)    => sOddrOutAbstract_5
    );
    

    

    
end Behavior;